/*
** Module   : AlertBoxImpl.java
** Abstract :
**
** Copyright (c) 2011-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 SIY 20111122 Extracted from AlertBox
** 002 SIY 20111221 Fixed constructor visibility.
** 003 CA  20140805 Do not allow ChUI impl classes access to APIs outside of ChuiOutputManager.
**                  Replaced size() API with dimension().
** 004 HC  20140911 GUI coordinate support - data types refactoring.
** 005 CA  20141119 Refactored drawing code to enable multiple OS windows, for GUI support.  Misc
**                  changes/fixes related to drawing on multiple windows.
** 006 VIG 20150407 Typified Window.getDefaultWindow method call.
*/

package com.goldencode.p2j.ui.chui;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.chui.driver.*;
import com.goldencode.p2j.ui.client.widget.*;
import com.goldencode.p2j.util.*;

/**
 * This class is an specific implementation of the AlertBox for CHUI.
 */
public class AlertBoxImpl
extends AlertBox<ChuiOutputManager>
{
   /**
    * Creates a modal dialog with a title and with a frame owner. This
    * constructor creates an AlertBox with a message body which is
    * represented by a content array. The alert box can contain a user
    * defined button set and the specified title. Each line of the
    * message including any number of empty lines (skip) are elements of the
    * content array.  Text elements of the content array must be strings.
    * In addition, there may be any number of <code>skip</code> elements 
    * which are encoded as a Java <code>Integer</code> instance where the
    * integral value is equivalent to the number of lines to skip.
    * <p>
    * The value of the <code>var</code> being edited determines the default
    * button (<code>true</code> is the first button, <code>false</code>
    * is the second button and for a 3-button dialog, <code>unknown</code> is 
    * the third button).
    * <p>
    * The result of the user's selection can be read via {@link #getResult}.
    * The mapping of the result value to a button follows the same mapping
    * as used for setting the default button (see above).
    *
    * @param    content
    *           Array of elements which form alert box content.
    * @param    buttonSet
    *           Button set type to be constructed for this AlertBox.
    * @param    var
    *           The variable to be edited.  The initial value sets the 
    *           default button and the resulting value is changed by the
    *           user's press of a button.
    * @param    title
    *           AlertBox title to be rendered at the top of the one line
    *           border.
    * @param    color
    *           Color scheme to use.
    */
   AlertBoxImpl(Object[] content,
                       int      buttonSet,
                       logical  var,
                       String   title,
                       Color    color)
   {
      super(content, buttonSet, var, title, color);
   }
   
   /**
    * Hide/show widget.
    * <p>
    * If the alert box is being shown and not yet attached to a window, attached it to the default
    * window.
    * 
    * @param   visible
    *          New visible state.
    */
   @Override
   public void setVisible(boolean visible)
   {
      if (visible && parent() == null)
      {
         setParent(Window.<ChuiOutputManager>getDefaultWindow());
      }

      super.setVisible(visible);
   }
   
   /**
    * The main AlertBox renderer. Draws the contents of the AlertBox and
    * requests the focus to the one of the button within the container.
    */
   @Override
   public void draw()
   {
      // draw the enclosing frame
      Color dc = alertColor;

      clear(location(), dimension(), dc);
      screen().boxToScreen(location(), dimension(), dc);
                          
      // draw the horizontal line
      screen().at(location().offset(2, dimension().height - 3))
              .hLine(dimension().width - 4, dc);

      // draw the title
      if (hasTitle())
      {
         // compute the title's X position, depending if was trimmed or not
         int x = forceTitleLeftAlign ? 0 
                                      : (int) (dimension().width - title().length()) / 2;
         
         Point offsetPoint = location().offset(x, 0);
         screen().at(offsetPoint).append(title(), dc);
      }
      
      // draw all the contained components
      Widget<ChuiOutputManager> focus = null;
      
      for (Widget<ChuiOutputManager> widget : widgets())
      {
         if (widget.isVisible())
         {
            if (widget.hasFocus())
               focus = widget;
            else
               widget.draw();
         }
      }
      
      if (focus != null)
         focus.draw();
   }
}
