/*
** Module   : ButtonImpl.java
** Abstract :
**
** Copyright (c) 2011-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 SIY 20111122 Extracted from Button.
** 002 SIY 20111221 Fixed constructor visibility.
** 003 SVL 20120214 Fixed mnemonic processing.
** 004 GES 20140627 Changed color attribute method names to match naming standards.
** 005 CA  20140805 Do not allow ChUI impl classes access to APIs outside of ChuiOutputManager.
** 006 HC  20140911 GUI coordinate support - data types refactoring.
** 007 CA  20140926 Added shared widget configuration support, to allow GUI/ChUI concrete
**                  implementation for the same widget ID.  Refactored the widget configuration
**                  classes: all fields were made public; these classes need to be as dumb as
**                  possible, all logic related to setting/getting a certain field should be at
**                  the widget or at the caller. Refs #2254
** 008 VIG 20150406 Added pop-up menu key listener.
*/

package com.goldencode.p2j.ui.chui;

import com.goldencode.p2j.ui.ButtonConfig;
import com.goldencode.p2j.ui.Color;
import com.goldencode.p2j.ui.WidgetConfig;
import com.goldencode.p2j.ui.WidgetId;
import com.goldencode.p2j.ui.client.Button;
import com.goldencode.p2j.ui.client.Point;
import com.goldencode.p2j.ui.client.UiUtils;
import com.goldencode.p2j.ui.client.chui.MenuChuiImpl;
import com.goldencode.p2j.ui.client.chui.driver.ChuiOutputManager;
import com.goldencode.p2j.ui.client.widget.Container;

/**
 * Specific implementation of a "push button" with a text label for CHUI.
 */
@SuppressWarnings("unchecked")
public class ButtonImpl
extends Button<ChuiOutputManager>
{
   /**
    * This constructor creates a button with a given text label.
    * 
    * @param   id
    *          The widget's ID.
    * @param   owner
    *          The parent container, frame or dialog which will include this
    *          button.
    * @param   text
    *          The string to be used as the button text.
    */
   ButtonImpl(WidgetId id, Container<ChuiOutputManager> owner, String text) 
   {
      super(id, owner, text);
   }

   /**
    * Special constructor used to restore component using given configuration.
    * 
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Configuration data.
    */
   public ButtonImpl(WidgetId id, WidgetConfig cfg)
   {
      super(id, (ButtonConfig) cfg);
   }
   
   /**
    * Replacement API for the logic which should exist in the widget's c'tor, but the logic 
    * can't be executed as it depends on the widget being registered into the official registry.
    * <p>
    * This API is called after the widget is created and added to the registry.
    * 
    * @param    id
    *           The widget's ID.
    * @param    cfg
    *           The config used to initialize this widget.
    */
   @Override
   public void initialize(WidgetId id, WidgetConfig cfg)
   {
      super.initialize(id, cfg);
      
      addKeyListener(new MenuChuiImpl.PopupKeyListener());
   }
   
   /**
    * Draw the button. Called by this Button's parent container.
    */
   @Override
   public void draw()
   {
      // get the absolute origin of this component
      Point origin = screenLocation();

      screen().at(origin);
      Color color = UiUtils.getDColor(this);

      //  "highlighted" flag tells whether this button needs emphasis
      if (super.isEnabled() && highlighted())
      {
         // Progress behaves somewhat strange here:
         // 1. Focused blinking button does not blink
         // 2. Focused underlined button blinks(!) and does not use reverse
         //    attribute.
         if (color.hasUnderline())
         {
            color = color.withoutUnderline().withBlink();
         }
         else
         {
            color = color.withReverse().withoutBlink();
         }
      }
      
      // obtain the full text of the button's label
      String ct = textLabel();
      
      // calculate the total size of the button
      // in chui char coordinates are always whole numbers
      int bSize = (int) width();
      
      // calculate the max size of the text part
      int tSize = bSize - 2;
      
      // calculate the actual text length (may be shorter or longer than the
      // text size)
      int tActual = ct.length(); 
      
      StringBuffer sb = new StringBuffer();
      
      // there is always the left angle bracket since the size must be at
      // least 1 (but it may only be 1... and this would be the only output)
      sb.append("<");
      
      if (tSize > 0)
      {
         if (tSize < tActual)
         {
            // right truncate the text
            sb.append(ct.substring(0, tSize));
         }
         else if (tSize == tActual)
         {
            // we are just big enough to hold the full text
            sb.append(ct);
         }
         else
         {
            // we must pad with spaces (if there an even number of spaces
            // then the left and right sides will have the same number, if
            // odd then the right side will have the extra space)
            
            int spaces = tSize - tActual;
            int left   = spaces / 2; 
            int right  = (spaces % 2) > 0 ? left + 1 : left;
            
            for (int i = 0; i < left; i++)
            {
               sb.append(" ");
            }
            
            sb.append(ct);
            
            for (int i = 0; i < right; i++)
            {
               sb.append(" ");
            }
         }
      }
      
      // any button size of 2 or more will always have a right angle
      // bracket, even if it has no text
      if (bSize > 1)
      {
         sb.append(">");
      }

      // actually draw the core button
      screen().append(sb.toString(), color);

      // mnemonic processing
      if (underscore >= 0 && (underscore + 1) < bSize)
      {
         color = UiUtils.getDColor(this);

         // note that this draws in the non-reversed form whether selected
         // or not (this is the equivalent of a "highlight" in the selected
         // mode because the rest of the button was drawn in reverse video

         if (!highlighted())
            color = color.withUnderline();

         String mnemonic = ct.substring(underscore, underscore + 1);
         
         screen().at(origin.offset(1 + underscore, 0))
                 .append(mnemonic, color);
      }
   }
   
   /**
    * Request focus.
    */
   @Override
   public void requestFocus()
   {
      super.requestFocus();

      screen().at(screenLocation().offset(1,0));
   }
}
