/*
** Module   : Menu.java
** Abstract : Menu abstract client implementation
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------------Description-----------------------------
** 001 VIG 20150116 Created initial version.
** 002 VIG 20150318 Added getFocusedChild() method and overridden focusWorker() method.
*/
package com.goldencode.p2j.ui.client;

import java.util.List;

import com.goldencode.p2j.ui.ConfigManager;
import com.goldencode.p2j.ui.SubMenuConfig;
import com.goldencode.p2j.ui.WidgetConfig;
import com.goldencode.p2j.ui.WidgetId;
import com.goldencode.p2j.ui.client.widget.AbstractContainer;
import com.goldencode.p2j.ui.client.widget.Container;
import com.goldencode.p2j.ui.client.widget.Widget;
import com.goldencode.p2j.util.Iterables;

/**
 * <code>Menu</code> client abstract implementation class. 
 */
public abstract class SubMenu<O extends OutputManager<?>>
extends AbstractContainer<O>
{
   /** Menu attribute container. */
   private SubMenuConfig config = null; 

   /**
    * Special constructor used to restore component using given configuration.
    * 
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Configuration data.
    */
   public SubMenu(WidgetId id, SubMenuConfig cfg)
   {
   }
   
   /**
    * Replacement API for the logic which should exist in the widget's c'tor, but the logic 
    * can't be executed as it depends on the widget being registered into the official registry.
    * <p>
    * This API is called after the widget is created and added to the registry.
    * 
    * @param    id
    *           The widget's ID.
    * @param    cfg
    *           The config used to initialize this widget.
    */
   @Override
   public void initialize(WidgetId id, WidgetConfig cfg)
   {
      config = ConfigManager.getInstance().getActiveConfig(id);
      if (config == null)
      {
         config = new SubMenuConfig(id);
      }
      ConfigManager.getInstance().replaceConfig(config, cfg);
   }   

   /**
    * Provide access to the attributes stored in the widget.
    * 
    * @return  Reference to attribute container.
    */
   @SuppressWarnings("unchecked")
   @Override
   public SubMenuConfig config()
   {
      return config;
   }
   
   /**
    * Gets sub-menu title width (with '->' if sub-menu is in other sub-menu or in popup menu)
    * 
    * @return   width of sub-menu title
    */
   public double titleWidth()
   {
      double w = config().title.length();
      
      if (!isMenubar())
      {
         w += 3; // " ->" will be added
      }
      
      return w;
   }

   /**
    * Hides sub-menu body.
    */
   public void hideBody()
   {
      
   }
   
   /**
    * Get first focused sub-menu child (menu-iter or other sub-menu)
    * 
    * @return   MenuItem or SubMenu
    */
   protected Widget<?> getFocusedChild()
   {
      for(Widget<?> w : widgets())
      {
         if (w.hasFocus())
         {
            return w;
         }
      }
      
      return null;
   }

   /**
    * Search for the next or previous focusable widget in the container and
    * set focus on it.
    *
    * @param   direction
    *          Widget search direction.
    */
   protected void focusWorker(boolean direction)
   {
      List<Widget<O>> widgets = getWidgetsList();
      Iterable<Widget<O>> iter = (direction) ?
                                 Iterables.directFrom(widgets, focus()) :
                                 Iterables.reverseFrom(widgets, focus());
                                     
       for (Widget<O> widget : iter)
       {
          if (checkWidget(widget, direction))
             return;
       }
    
       // cyclic transition to the first or last element
       Widget<O> nextWidget = direction ? widgets.get(0) : widgets.get(widgets.size() - 1);
       checkWidget(nextWidget, direction);
   }

   /**
    * Check if this sub-menu is right inside MENUBAR (first level)
    * 
    * @return  <code>true</code> if yes.
    */
   protected boolean isMenubar()
   {
      Container<?> parent = parent();
      return parent instanceof Menu && !((Menu<?>)parent).config().popupOnly;
   }

}
