/*
** Module   : MenuChuiImpl.java
** Abstract : CHUI client implementation of Menu.
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------------Description-----------------------------
** 001 VIG 20150116 Created initial version.
** 002 VIG 20150226 Added processKeyEvent method.
*/
package com.goldencode.p2j.ui.client.chui;

import com.goldencode.p2j.ui.Keyboard;
import com.goldencode.p2j.ui.MenuConfig;
import com.goldencode.p2j.ui.WidgetConfig;
import com.goldencode.p2j.ui.WidgetId;
import com.goldencode.p2j.ui.client.Dimension;
import com.goldencode.p2j.ui.client.Menu;
import com.goldencode.p2j.ui.client.MenuItem;
import com.goldencode.p2j.ui.client.Point;
import com.goldencode.p2j.ui.client.SubMenu;
import com.goldencode.p2j.ui.client.Window;
import com.goldencode.p2j.ui.client.border.Border;
import com.goldencode.p2j.ui.client.border.LineBorder;
import com.goldencode.p2j.ui.client.chui.driver.ChuiOutputManager;
import com.goldencode.p2j.ui.client.event.EventManager;
import com.goldencode.p2j.ui.client.event.ItemEvent;
import com.goldencode.p2j.ui.client.event.KeyInput;
import com.goldencode.p2j.ui.client.event.listener.KeyListener;
import com.goldencode.p2j.ui.client.widget.AbstractWidget;
import com.goldencode.p2j.ui.client.widget.Widget;

/**
 * <code>Menu</code> CHUI client implementation class. 
 */
@SuppressWarnings("unchecked")
public class MenuChuiImpl
extends Menu<ChuiOutputManager>
{
   /** Border around the panel. */
   private Border<ChuiOutputManager> border = null;
   
   /**
    * Special constructor used to restore component using given configuration.
    * 
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Configuration data.
    */
   public MenuChuiImpl(WidgetId id, WidgetConfig cfg)
   {
      super(id, (MenuConfig) cfg);
      
      MenuConfig mcfg = (MenuConfig) cfg;
      if (mcfg.popupOnly)
      {
         border = new LineBorder();
      }
   }
   
   /**
    * Drawing worker method.
    */
   public void drawPanel()
   {
      screen().clear(WidgetId._DEFAULT_WINDOW_ID, 
               location(), 
               dimension(false), 
               screen().getInteractiveOutput().widgetColor(this));

      if (border != null)
         border.drawBorder(this, dimension(false));
   }

   /**
    * Draw the peer container. The methods calls the draw() method for
    * ScrollPane instance for the peer container. In addition the required
    * clipping rectangle is calculated for ChildFrame containers because they
    * must be rendered inside the parent container.
    * <p>
    * Drawing for the redirected terminal differs. Labels/headers are drawn
    * only once per page, instead of the usual once per menu.
    */
   @Override
   public void draw()
   {
      drawPanel();
      super.draw();
   }

   /**
    * Get children location for one child.
    * 
    * @param   child
    *          Child component.
    *          
    * @return  Point - parents location
    */
   @Override
   public Point childrenLocation(Widget<ChuiOutputManager> c)
   {
      Point bodyLoc = location();
      return new Point(bodyLoc.x + 1, bodyLoc.y); // border
   }
   
   /**
    * Hide/show widget.
    * <p>
    * If the menu is being shown and not yet attached to a window, attached it to the default
    * window.
    * 
    * @param   visible
    *          New visible state.
    */
   @Override
   public void setVisible(boolean visible)
   {
      setDisplayed(visible);
      super.setVisible(visible);
      
      if (visible && parent() == null)
      {
         setParent(Window.<ChuiOutputManager>getDefaultWindow());
      }
      
      for(Widget<ChuiOutputManager> w : widgets())
      {
         w.setVisible(true);
      }

   }

   /**
    * Replace original version to take into account P2J specific behavior.
    * 
    * @see  com.goldencode.p2j.ui.client.widget.Widget#dimension()
    */
   @Override
   public Dimension dimension()
   {
      return dimension(true);
   }

   /**
    * Process key event.
    *
    * @param   event
    *          Key event to process.
    */
   @Override
   public void processKeyEvent(KeyInput event)
   {
      if (event.isConsumed())
      {
         return;
      }
      
      // TODO add MENU-DROP event
      // EventManager.postEvent(new KeyInput(this, EventType.MENU_DROP_EVENT, Key.VK_ENTER));

      // mnemonic 
      if (event.keyChar() != KeyInput.CHAR_UNDEFINED)
      {
         char ch = Character.toLowerCase(event.keyChar());
         for(Widget<?> w : getWidgetsList())
         {
            if (w instanceof MenuItem)
            {
               MenuItem<?> m = (MenuItem<?>) w;
               if (m.config().mnemonic != null && // mnemonic 
                   Character.toLowerCase(m.config().mnemonic.charValue()) == ch) 
               {
                  EventManager.postEvent(new ItemEvent(m));
                  return;
               }
            }
         }
      }
      // accelerator
      else if (event.isRealKey())
      {
         int code = event.keyCode();
         for(Widget<?> w : getWidgetsList())
         {
            if (w instanceof MenuItem)
            {
               MenuItem<?> m = (MenuItem<?>) w;
               if (m.config().accelerator != null && 
                   Keyboard.keyCode(m.config().accelerator) == code)
  
               {
                  EventManager.postEvent(new ItemEvent(m));
                  return;
               }
            }
         }
         
      }
      else
      {
         super.processKeyEvent(event);
      }
   }
   
   /**
    * Lay out container.
    */
   @Override
   public void doLayout()
   {
      setLocation(calcLocation());
      super.doLayout();
      
      Widget<ChuiOutputManager>[] w = widgets();
      double x=0;
      
      for(int i=0; i<w.length; i++)
      {
         if (!config().popupOnly) // menubar
         {
            w[i].setLocation(new Point(x, 0));
            x += 1 + w[i].width();
         }
         else
         {
            w[i].setLocation(new Point(1, i + 1));
         }
         
         if(w[i] instanceof SubMenu)
         {
            ((SubMenu<?>)w[i]).doLayout();
         }
      }
   }
   
   /**
    * Calculates MENU location relative to its parent if it is POPUP-ONLY or 
    * [0, 0] if it is MENUBAR.
    * 
    * @return   point of location relative to its parent.
    */
   private Point calcLocation()
   {
      if (config().popupOnly)
      {
         Widget<?> owner = screen().getRegistry().getComponent(config().ownerId);
         if (owner != null)
         {
            AbstractWidget<?> awOwner = (AbstractWidget<?>) owner;
            return new Point(awOwner.location().x + 1, awOwner.location().y + awOwner.height() + 1);
         }
      }
      
      return new Point(0, 0); // menubar or owner is not set
   }
   
   /**
    * Replace original version to take into account P2J specific behavior.
    * 
    * @see  com.goldencode.p2j.ui.client.widget.Widget#dimension()
    * 
    * @param   includeNested
    *          <code>true</code> if we should process nested sub-menus
    */
   private Dimension dimension(boolean includeNested)
   {
      double height = SubMenuChuiImpl.getMaxItemHeight(this, includeNested);
      double width = SubMenuChuiImpl.getMaxItemWidth(this, includeNested);
      
      if (!config().popupOnly)
      {
         double menubarWidth = 0; 
         for(Widget<?> w : widgets())
         {
            AbstractWidget<?> aw = (AbstractWidget<?>) w;
            menubarWidth += aw.width() + 1; 
         }
         width = Math.max(menubarWidth, width);
      }
      
      return new Dimension(width + 4, height); 
   }
   
   /**
    * Key listener, intended to show popup menu. Used for different components. 
    */
   public static class PopupKeyListener 
   implements KeyListener
   {

      /**
       * This method is invoked when key is pressed.
       * 
       * @param   event
       *          Event which triggered this call.
       */
      @Override
      public void onKeyPressed(KeyInput evt)
      {
         // Should we process TriggerResult here?
         
         Widget<?> src = evt.source();
         int action = (evt == null ? KeyInput.CHAR_UNDEFINED  : evt.actionCode());

         // an active pop-up menu key
         if (action == Keyboard.KA_DEFAULT_POP_UP && src != null)
         {
            int menuId = src.config().popupMenuId;
            if (menuId != -1)
            {
               MenuChuiImpl menu = (MenuChuiImpl) src.screen().getRegistry().getComponent(menuId);
               if (menu != null)
               {
                  if (menu.parent() == null)
                  {
                     Window.<ChuiOutputManager>getDefaultWindow().getContentPane().add(menu);
                  }
                  
                  if (!menu.isVisible())
                  {
                     menu.setVisible(true);
                     menu.firstFocus();
                  }
                  
                  menu.draw();
                  
                  evt.consume();
               }
            }
         }
      }

      /**
       * This method is invoked when key is typed.
       * 
       * @param   event
       *          Event which triggered this call.
       */
      @Override
      public void onKeyTyped(KeyInput event)
      {
         // Nothing
      }
   }
}
