/*
** Module   : MenuChuiImpl.java
** Abstract : CHUI client implementation of Menu.
**
** Copyright (c) 2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------------Description-----------------------------
** 001 VIG 20150116 Created initial version.
** 002 VIG 20150225 Added event processing and drawing methods.
*/
package com.goldencode.p2j.ui.client.chui;

import java.util.List;

import com.goldencode.p2j.ui.Color;
import com.goldencode.p2j.ui.Keyboard;
import com.goldencode.p2j.ui.MenuItemConfig;
import com.goldencode.p2j.ui.WidgetConfig;
import com.goldencode.p2j.ui.WidgetId;
import com.goldencode.p2j.ui.chui.ThinClient;
import com.goldencode.p2j.ui.client.Dimension;
import com.goldencode.p2j.ui.client.Menu;
import com.goldencode.p2j.ui.client.MenuItem;
import com.goldencode.p2j.ui.client.Point;
import com.goldencode.p2j.ui.client.Rectangle;
import com.goldencode.p2j.ui.client.SubMenu;
import com.goldencode.p2j.ui.client.chui.driver.ChuiOutputManager;
import com.goldencode.p2j.ui.client.event.EventManager;
import com.goldencode.p2j.ui.client.event.EventType;
import com.goldencode.p2j.ui.client.event.ItemEvent;
import com.goldencode.p2j.ui.client.event.Key;
import com.goldencode.p2j.ui.client.event.KeyInput;
import com.goldencode.p2j.ui.client.widget.AbstractContainer;
import com.goldencode.p2j.ui.client.widget.Container;
import com.goldencode.p2j.ui.client.widget.Widget;

/**
 * <code>Menu</code> CHUI client implementation class. 
 */
@SuppressWarnings("unchecked")
public class MenuItemChuiImpl
extends MenuItem<ChuiOutputManager>
{
   /**
    * Special constructor used to restore component using given configuration.
    * 
    * @param   id
    *          The widget's ID.
    * @param   cfg
    *          Configuration data.
    */
   public MenuItemChuiImpl(WidgetId id, WidgetConfig cfg)
   {
      super(id, (MenuItemConfig) cfg);
   }

   /**
    * Draw the peer container. The methods calls the draw() method for
    * ScrollPane instance for the peer container. In addition the required
    * clipping rectangle is calculated for ChildFrame containers because they
    * must be rendered inside the parent container.
    * <p>
    * Drawing for the redirected terminal differs. Labels/headers are drawn
    * only once per page, instead of the usual once per menu.
    */
   @Override
   public void draw()
   {
      Color dColor = config().dcolor == null ? Color.NORMAL : config().dcolor;
      
      Rectangle rect = bounds().translate(parent().childrenLocation(this));
      
      drawTitle(new Point(rect.left(), rect.top()), dColor);
   }

   /**
    * Get widget dimension.
    * 
    * @return  widget dimension.
    */
   @Override
   public Dimension dimension()
   {
      return new Dimension(width(), 1);
   }

   /**
    * Get widget height.
    * 
    * @return  widget height.
    */
   @Override
   public double height()
   {
      return 1;
   }

   /**
    * Get widget minimum size.
    * 
    * @return  widget minimum size.
    */
   @Override
   public Dimension minimumSize()
   {
      return dimension();
   }

   /**
    * Get widget width.
    * 
    * @return  widget width.
    */
   @Override
   public double width()
   {
      MenuItemConfig cfg = config();
      
      if (!cfg.isNormal())
      {
         // set width to longest neighbor's width.
         double width = 0;
         for (Widget<?> wdg : parent().widgets())
         {
            // prevent infinite loop
            if (wdg.equals(this))
            {
               continue;
            }
            
            // prevent infinite loop
            if (wdg instanceof MenuItem && !((MenuItem<?>)wdg).config().isNormal())
            {
               continue;
            }
            
            double w = wdg instanceof SubMenu ? ((SubMenu<?>) wdg).titleWidth() : wdg.width();
            
            if (width < w)
            {
               width = w;
            }
         }
         
         return width;
      }
      
      int w = cfg.title.length();
      
      if (cfg.title.contains("&"))
         w--;
      
      if (cfg.toggleBox)
         w++;
      
      return w;
   }

   /**
    * Process key event.
    * 
    * @param   event
    *          Key event to process.
    */
   @Override
   public void processKeyEvent(KeyInput event)
   {
      if (event.isConsumed())
      {
         return;
      }

      int key = event.keyCode();
      int actionCode = event.actionCode();
      
      // menu-item choosing
      if (key == Key.VK_ENTER ||
         (isMenubar() && actionCode == Keyboard.KA_CURSOR_DOWN) ||       
         (isMenubar() && actionCode == Keyboard.KA_CURSOR_UP))
      {
         KeyInput itmEvt;
         MenuItemConfig cfg = config();
         if (cfg.toggleBox)
         {
            cfg.checked = !cfg.checked;
            cfg.setModified(true);
            itmEvt = new KeyInput(this, EventType.KEY_PRESSED, Keyboard.SE_VALUE_CHANGED, true);
            EventManager.postEvent(itmEvt);
         }
         else
         {
            //itmEvt = new KeyInput(this, EventType.KEY_PRESSED, Keyboard.SE_CHOOSE, true);
            EventManager.postEvent(new ItemEvent(this));
         }

         return;
      }

      // next focus from menu-item
      if ((!isMenubar() && actionCode == Keyboard.KA_CURSOR_DOWN) || 
          (isMenubar() && actionCode == Keyboard.KA_CURSOR_RIGHT))
      {
         ThinClient.nextFocus(this);
         parent().repaint();
         return;
      }

      // previous focus from menu-item
      if ((!isMenubar() && actionCode == Keyboard.KA_CURSOR_UP) || 
          (isMenubar() && actionCode == Keyboard.KA_CURSOR_LEFT))
      {
         ThinClient.previousFocus(this);
         parent().repaint();
         return;
      }

      // hiding sub-menu from menu-item
      if ((!isMenubar() && actionCode == Keyboard.KA_CURSOR_RIGHT) || 
          (!isMenubar() && actionCode == Keyboard.KA_CURSOR_LEFT))
      {
         if (parent() instanceof SubMenu)
         {
            ((SubMenu<?>)parent()).hideBody();
            return;
         } 
         else if (((Menu<?>)parent()).config().popupOnly) // pop-up menu
         {
            // TODO hide pop-up menu
         }
      }
         
      // mnemonic
      if (event.keyChar() != KeyInput.CHAR_UNDEFINED || 
          event.isRealKey() ||
          actionCode == Keyboard.SE_VALUE_CHANGED ||
          actionCode == Keyboard.SE_CHOOSE)
      {
         parent().processKeyEvent(event);
      }
      else
      {
         super.processKeyEvent(event);
      }
      
   }

   /**
    * Check if widget is focus traversable. SKIP and RULE aren't traversable.
    * 
    * @return  <code>true</code> if widget is focus traversable.
    */
   @Override
   public boolean focusTraversable()
   {
      return config().isNormal() ? super.focusTraversable() : false;
   }
   
   /**
    * Check if widget needs in cursor to be on when having the input focus.  By default this
    * method always returns <code>true</code> so it needs to be overwritten if other return value
    * is required for particular widget implementation class.
    * 
    * @return  <code>true</code> as the default value.
    */
   @Override
   public boolean forceCursorOnFocus()
   {
      return false;
   }

   /**
    * Make some layout settings for this component such as location and dimension.
    */
   public void doLayout()
   {
      setLocation(calcLocation());
   }

   /**
    * Trick, that lets to correct parent location in cases, when it needed,
    * for example for menus.
    * 
    * @param    parent
    *           Parent component.
    *           
    * @return   corrected parent location.
    */
   @Override
   protected Point prepParentLocation(Widget<?> parent)
   {
      Point loc = parent.location();
      if (parent instanceof SubMenu)
      {
         loc.x += 1;
         loc.y += 2;
      }
      return loc;
   }
   
   /**
    * Calculates relative item position.
    * 
    * @return  new Point of this item.
    */
   private Point calcLocation()
   {
      AbstractContainer<ChuiOutputManager> parent = (AbstractContainer<ChuiOutputManager>) parent();      
      
      List<Widget<ChuiOutputManager>> widgets = parent.getWidgetsList();

      int idx = widgets.indexOf(this);
      
      if (idx == -1) 
      {
         throw new IllegalStateException("MENU-ITEM is not contained in its parents widgets list");
      }
      
      double x = 0;
      double y = 0;
      
      if (parent instanceof Menu && !((Menu<?>)parent).config().popupOnly)
      {
         for(Widget<?> w : widgets)
         {
            if (w.equals(this))
               break;
            x += 1 + w.width();
         }
      }
      else
      {
         x += 2;
         y += 2 + idx;
      }
      
      return new Point(x, y);
   }

   /**
    * Draws menu-item title, drawing mnemonic character as underlined.
    * 
    * @param   p
    *          Title point.  
    * @param   dColor
    *          Color of the title.
    */
   private void drawTitle(Point p, Color dColor)
   {
      if (hasFocus())
      {
         dColor = dColor.withReverse();
      }
      
      // SKIP or RULE
      if (config().isSkip())
      {
         Character skipRuleChar = ' '; 
         StringBuffer buf = new StringBuffer();
         for(int i=0; i<width(); i++)
            buf.append(skipRuleChar);
         screen().at(p).append(buf.toString(), Color.NORMAL);
         return; 
      }

      // RULE
      if (config().isRule())
      {
         screen().at(p).hLine(width(), Color.NORMAL);
         return;
      }
      
      // NORMAL
      String title = config().title;
      if (config().toggleBox)
      {
         title = (config().checked ? ">" : " ") + title;
      }
      
      Point tp = new Point(p);
      Color mnemonicColor = dColor.withUnderline().withoutReverse();
      
      int ampIdx = title.indexOf('&');
      boolean amp = ampIdx != -1; 
      
      // if & is not specified use the first letter as mnemonic
      if (!amp) ampIdx = config().toggleBox ? 1 : 0;

      String leftPart = title.substring(0, ampIdx);
      screen().at(tp).append(leftPart, dColor);
      
      tp.x += ampIdx;
      // "eat" the ampersand
      if (amp) ampIdx++;
      screen().at(tp).append(title.substring(ampIdx, ampIdx + 1), mnemonicColor);

      tp.x += 1;
      screen().at(tp).append(title.substring(ampIdx + 1), dColor);
   }

   /**
    * Check if this sub-menu is right inside MENUBAR (first level)
    * 
    * @return  <code>true</code> if yes.
    */
   private boolean isMenubar()
   {
      Container<ChuiOutputManager> parent = parent();
      return parent instanceof Menu && !((Menu<?>)parent).config().popupOnly;
   }
}
